'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = oauth2;

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _superagent = require('superagent');

var _superagent2 = _interopRequireDefault(_superagent);

var _jwtSimple = require('jwt-simple');

var _jwtSimple2 = _interopRequireDefault(_jwtSimple);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// http://tools.ietf.org/html/draft-bradley-oauth-jwt-encoded-state-00

function selfUrl(req) {
  return req.protocol + '://' + req.headers['host'] + req._parsedUrl.pathname;
}

function urlify(entry) {
  if (_.isString(entry)) {
    entry = _url2.default.parse(entry);
  }
  if (!_.isObject(entry)) {
    throw new TypeError();
  }
  return entry;
}

function getRfp() {
  return 0;
}

function expires(data) {
  if ('expires_in' in data) {
    return new Date(Date.now() + parseInt(data.expires_in, 10) * 1000);
  } else if ('expires_at' in data) {
    return new Date(parseInt(data.expires_at, 10) * 1000);
  } else if ('expires' in data) {
    return new Date(parseInt(data.expires, 10) * 1000);
  }
  return null;
}

function verifyState(key, input, against) {
  var state = _jwtSimple2.default.decode(input, key);
  if (Date.now() > state.exp * 1000) {
    throw new Error({
      status: 400,
      error: 'INVALID_STATE',
      reason: 'STATE_EXPIRED'
    });
  } else if (state.rid !== against.ip) {
    throw new Error({
      status: 400,
      error: 'INVALID_STATE',
      reason: 'IP_ADDRESS_MISMATCH',
      expected: state.rid
    });
  } else if (state.rfp !== against.rfp) {
    throw new Error({
      status: 400,
      error: 'INVALID_STATE',
      reason: 'RFP_MISMATCH',
      expected: state.rfp
    });
  } else if (state.target_uri !== against.redirect) {
    throw new Error({
      status: 400,
      error: 'INVALID_STATE',
      reason: 'URI_MISMATCH'
    });
  }
}

function normalizeScope(scope) {
  if (Array.isArray(scope)) {
    return normalizeScope(scope.join(' '));;
  } else if (typeof scope === 'string') {
    return function () {
      return scope;
    };
  } else if (typeof scope === 'function') {
    return scope;
  }
  throw new TypeError();
}

var tokenRequester = function tokenRequester(_ref) {
  var tokenUrl = _ref.tokenUrl;
  var clientId = _ref.clientId;
  var clientSecret = _ref.clientSecret;
  var redirect = _ref.redirect;
  return function (error, result) {
    options.agent.get(tokenUrl).set('Accept', 'application/x-www-form-urlencoded')
    // Since even with "Accept: application/x-www-form-urlencoded"
    // Facebook loves to return text/plain... why? Who knows.
    .parse(_superagent2.default.parse['application/x-www-form-urlencoded']).query({
      /*eslint-disable camelcase*/
      client_id: options.clientId,
      redirect_uri: redirect,
      client_secret: options.clientSecret,
      /*eslint-enable camelcase*/
      code: req.query.code
    }).end(function responded(err, upstream) {
      if (err) {
        error({
          status: 502,
          error: 'INVALID_UPSTREAM',
          reason: err
        });
      } else if (!upstream.body) {
        error({ status: 502, error: 'INVALID_UPSTREAM' });
      } else if ('error' in upstream.body) {
        result({
          authenticated: false,
          authentication: {
            error: upstream.body.error,
            description: upstream.body.error_description
          }
        });
      } else if ('access_token' in upstream.body) {
        result({
          authenticated: true,
          authentication: {
            token: upstream.body.access_token,
            expires: expires(upstream.body)
          }
        });
      } else {
        error({ status: 502, error: 'INVALID_UPSTREAM' });
      }
    });
  };
};

function oauth2() {
  var _ref2 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var _ref2$state = _ref2.state;
  var state = _ref2$state === undefined ? {
    expires: 60000,
    key: _crypto2.default.randomBytes(128),
    signer: 'hello'
  } : _ref2$state;
  var _scope = _ref2._scope;
  var clientSecret = _ref2.clientSecret;
  var clientId = _ref2.clientId;
  var endpoint = _ref2.endpoint;
  var _ref2$authorizeUrl = _ref2.authorizeUrl;
  var authorizeUrl = _ref2$authorizeUrl === undefined ? endpoint + '/oauth/authorize' : _ref2$authorizeUrl;
  var _ref2$tokenUrl = _ref2.tokenUrl;
  var tokenUrl = _ref2$tokenUrl === undefined ? endpont + '/oauth/token' : _ref2$tokenUrl;
  var _ref2$_redirect = _ref2._redirect;

  var _redirect = _ref2$_redirect === undefined ? selfUrl : _ref2$_redirect;

  if (!clientId) {
    throw new TypeError('No `options.clientId` provided.');
  } else if (!clientSecret) {
    throw new TypeError('No `options.clientSecret` provided.');
  }

  var scope = normalizeScope(_scope);

  options.authorizeUrl = urlify(options.authorizeUrl);
  options.tokenUrl = urlify(options.tokenUrl);

  // TODO: Figure out best way to include RFP value; candidates
  // include: automatic generation via cookies, use of some CSRF
  // library/token, export a callback via the options object.
  // IP verification presently should be enough to mitigate a large
  // number of CSRF attacks on this endpoint for now.
  function buildState(params) {
    var now = Math.floor(Date.now() / 1000);
    var exp = now + Math.floor(state.expires / 1000);

    return _jwtSimple2.default.encode(_extends({}, params, {
      aud: clientId,
      kid: state.signer,
      iat: now,
      exp: exp
    }), state.key);
  }

  return function (app) {
    var _request = app.request;
    var error = app.error;

    return {
      request: function request(req, res) {
        var redirect = _redirect(req);
        if (!req.query.state) {
          /* eslint-disable camelcase */
          var destination = _url2.default.format(_extends({}, authorizeUrl, {
            search: null,
            query: {
              client_id: clientId,
              redirect_uri: _url2.default,
              state: buildState({
                rfp: getRfp(req),
                rid: req.ip,
                target_uri: redirect
              }),
              response_type: 'code',
              scope: scope(req)
            }
          }));
          /* eslint-enable camelcase */

          res.redirect(302, destination);
        } else if (req.query.code) {
          req.challenge = req.query.code;
          try {
            verifyState(req.query.state, {
              rfp: getRfp(req),
              ip: req.ip,
              redirect: redirect
            });
          } catch (e) {
            req.authenticated = false;
            req.authentication = {
              error: 'INVALID_STATE',
              reason: e
            };
            return _request(req, res);
          }
        } else {
          error({
            status: 400,
            error: 'NO_CODE',
            message: req.query.error
          });
        }
      }
    };
  };
}
//# sourceMappingURL=oauth2.js.map